#include <nori/integrators/integrator.h>
#include <nori/core/kdtree.h>

NORI_NAMESPACE_BEGIN

class Scene;

struct Photon {
    Point3f x;
    Vector3f w;
    Color3f phi;
    Normal3f n;
};

class PhotonMappingIntegrator : public Integrator {
public:
    PhotonMappingIntegrator(const PropertyList &props);

    void preprocess(const Scene *scene) override;

    void generatePhotonMap(const Scene *scene, Sampler *sampler);

    Color3f Li(const Scene *scene, Sampler *sampler, const Ray3f &ray) const override;

    std::string toString() const override;

private:
    const int m_photonCount;
    float m_radius2;
    const int m_samplesFG;
    const int m_samplesDI;

    std::vector<Photon> m_photonMap;
    
    typedef unsigned int PhotonMapIdx;
    typedef GenericKDTreeNode<Vector3f, PhotonMapIdx> PhotonKDTreeNode;
    PointKDTree<PhotonKDTreeNode> m_KDTree;
};

NORI_NAMESPACE_END