
#include <nori/integrators/pm.h>
#include <nori/core/scene.h>
#include <nori/warp/warp.h>
#include <nori/bsdfs/bsdf.h>
#include <nori/emitters/emitter.h>

NORI_NAMESPACE_BEGIN

PhotonMappingIntegrator::PhotonMappingIntegrator(const PropertyList &props):
    m_photonCount(props.getInteger("photonCount", 100)),
    m_radius2(props.getFloat("radius2", 0.05f)),
    m_samplesFG(props.getInteger("samplesFG", 100)),
    m_samplesDI(props.getInteger("samplesDI", 50))
{

}

void PhotonMappingIntegrator::preprocess(const Scene *scene) {
    // make radius2 independent of scene size
    m_radius2 *= scene->getBoundingBox().getExtents().norm();
    
    std::unique_ptr<Sampler> sampler = scene->getSampler()->clone();
    generatePhotonMap(scene, sampler.get());
}

void PhotonMappingIntegrator::generatePhotonMap(const Scene *scene, Sampler *sampler) {
    // ECSE689: TODO
}

Color3f PhotonMappingIntegrator::Li(const Scene *scene, Sampler *sampler, const Ray3f &ray) const {
    // ECSE689: TODO
    return 0.0f;
}

std::string PhotonMappingIntegrator::toString() const {
    return tfm::format(
        "PhotonMapper[\n"
        "photonCount = %d\n"
        "radius2 = %d\n"
        "samplesFG = %d\n"
        "samplesDI = %d\n"
        "]",
        m_photonCount,
        m_radius2,
        m_samplesFG,
        m_samplesDI
    );
}

NORI_REGISTER_CLASS(PhotonMappingIntegrator, "photon-map");
NORI_NAMESPACE_END